/*
 * Licensed to the Sakai Foundation (SF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The SF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package org.sakaiproject.nakamura.api.lite.accesscontrol;

/**
 * Thrown when access is denied. Contains a description of why.
 */
public class AccessDeniedException extends Exception {

    /**
	 * 
	 */
    private static final long serialVersionUID = -1838096370030304249L;
    private String objectPath;
    private String objectType;
    private String description;
    private String user;

    public AccessDeniedException(String objectType, String objectPath, String description,
            String user) {
        this.objectType = objectType;
        this.objectPath = objectPath;
        this.description = description;
        this.user = user;
    }

    @Override
    public String getMessage() {
        return "Denied " + user + " on " + objectType + ":" + objectPath + "  performing "
                + description;
    }

}
